package world;

import java.awt.*;


class Car {
	Vector2D position;
	double angle = 0;//Math.PI/8;
	double steering_angle = 0;//-Math.PI/16;
	double speed = 0;
	Car(Vector2D position, double angle){
		this.position = position;
		this.angle = angle;
	}

	void tick(){
		if (steering_angle == 0){
			position.x += speed * Math.cos(angle); 
			position.y += speed * Math.sin(angle);
		}
		else 
		{
			double radius = L / Math.tan(steering_angle);
			double da = speed / radius;
			double dx = radius * (1 - Math.cos(da));
			double dy = radius * Math.sin(da);
			
			Vector2D y_axis = new Vector2D(Math.cos(angle), Math.sin(angle));
			Vector2D x_axis = Vector2D.rotate90(y_axis);
			
			Vector2D diff = Vector2D.add(Vector2D.multiply(x_axis, dx),
					Vector2D.multiply(y_axis, dy));
			
			position.add(diff);
			angle += da;
		}
	}
	
	
	void move_forward(double radius, double travel_angle, double left_or_right, double front_or_back){
		//steering_angle = Math.abs(Math.atan2(radius, L)) * left_or_right;
		
		{
			double da = travel_angle;
			double dx = -radius * (1 - Math.cos(da)) * left_or_right;
			double dy = radius * Math.sin(da) * front_or_back;
			
			Vector2D y_axis = new Vector2D(Math.cos(angle), Math.sin(angle));
			Vector2D x_axis = Vector2D.rotate90(y_axis);
			
			Vector2D diff = Vector2D.add(Vector2D.multiply(x_axis, dx),
					Vector2D.multiply(y_axis, dy));
			
			position.add(diff);
			angle += da*left_or_right*front_or_back;
			
		}
	}
	void move_forward(double distance){
		Vector2D y_axis = new Vector2D(Math.cos(angle), Math.sin(angle));
		position.add(Vector2D.multiply(y_axis, distance));
	}
//	boolean pick(Vector2D v){
//		Vector2D[] vs = get_corners();
//
//		if (on_the_left(vs[0], vs[1], v))
//			return false;
//		if (on_the_left(vs[1], vs[2], v))
//			return false;
//		if (on_the_left(vs[2], vs[3], v))
//			return false;
//		if (on_the_left(vs[3], vs[0], v))
//			return false;
//		
//		return true;
//		
//	}
	Vector2D[] get_corners(){
		Vector2D y_axis = new Vector2D(Math.cos(angle), Math.sin(angle));
		Vector2D x_axis = Vector2D.rotate90(y_axis);
		
		Vector2D v0 = Vector2D.add(Vector2D.multiply(y_axis, L),
								Vector2D.multiply(x_axis, W/2));
		Vector2D v1 = Vector2D.add(Vector2D.multiply(y_axis, L),
				  				Vector2D.multiply(x_axis, -W/2));
		Vector2D v2 = Vector2D.add(Vector2D.multiply(y_axis, 0),
  								Vector2D.multiply(x_axis, -W/2));
		Vector2D v3 = Vector2D.add(Vector2D.multiply(y_axis, 0),
								Vector2D.multiply(x_axis, W/2));
		v0.add(position);
		v1.add(position);
		v2.add(position);
		v3.add(position);

		return new Vector2D[]{v0, v1, v2, v3};
	}
//	boolean on_the_left(Vector2D v0, Vector2D v1, Vector2D v){
//		Vector2D vec0 = new Vector2D(v0, v1);
//		Vector2D vec = new Vector2D(v0, v);
//		return Vector2D.cross_product(vec0, vec)<0;
//	}
	static int W = 80;
	static int L = 120;
	static double minimum_radius = 100;
	public void paint(Graphics g){
		g.setColor(Color.black);
		paint_rotated_rectangle(g, position, angle, W/2, L, 0);


		
		Vector2D y_axis = new Vector2D(Math.cos(angle), Math.sin(angle));
		Vector2D x_axis = Vector2D.rotate90(y_axis);
		
		Vector2D v0 = Vector2D.add(Vector2D.multiply(y_axis, L-15),
								Vector2D.multiply(x_axis, W/2-10));
		Vector2D v1 = Vector2D.add(Vector2D.multiply(y_axis, L-15),
				  				Vector2D.multiply(x_axis, -W/2+10));
		Vector2D v2 = Vector2D.add(Vector2D.multiply(y_axis, 0+15),
  								Vector2D.multiply(x_axis, -W/2+10));
		Vector2D v3 = Vector2D.add(Vector2D.multiply(y_axis, 0+15),
								Vector2D.multiply(x_axis, W/2-10));
		v0.add(position);
		v1.add(position);
		v2.add(position);
		v3.add(position);
		
		paint_rotated_rectangle(g, v0, angle+steering_angle, 5, 10, 10);
		paint_rotated_rectangle(g, v1, angle+steering_angle, 5, 10, 10);
		paint_rotated_rectangle(g, v2, angle, 8, 10, 10);
		paint_rotated_rectangle(g, v3, angle, 8, 10, 10);

	}	
	
	
	
	void paint_rotated_rectangle(Graphics g, Vector2D center, double angle, 
								 double w, double h0, double h1){
		
		Vector2D y_axis = new Vector2D(Math.cos(angle), Math.sin(angle));
		Vector2D x_axis = Vector2D.rotate90(y_axis);
		
		Vector2D v0 = Vector2D.add(Vector2D.multiply(y_axis, h0),
								Vector2D.multiply(x_axis, w));
		Vector2D v1 = Vector2D.add(Vector2D.multiply(y_axis, h0),
				  				Vector2D.multiply(x_axis, -w));
		Vector2D v2 = Vector2D.add(Vector2D.multiply(y_axis, -h1),
  								Vector2D.multiply(x_axis, -w));
		Vector2D v3 = Vector2D.add(Vector2D.multiply(y_axis, -h1),
								Vector2D.multiply(x_axis, w));
		v0.add(center);
		v1.add(center);
		v2.add(center);
		v3.add(center);
		
		g.drawLine((int)v0.x, (int)v0.y, (int)v1.x, (int)v1.y);
		g.drawLine((int)v1.x, (int)v1.y, (int)v2.x, (int)v2.y);
		g.drawLine((int)v2.x, (int)v2.y, (int)v3.x, (int)v3.y);
		g.drawLine((int)v3.x, (int)v3.y, (int)v0.x, (int)v0.y);
	}
	
	
	
	
	
	
	
	
	
	Vector2D get_head_vector(){
		return (new Vector2D(Math.cos(angle), Math.sin(angle)));
	}
	Vector2D get_local_coords(Vector2D v){
		Vector2D y_axis = new Vector2D(Math.cos(angle), Math.sin(angle));
		Vector2D x_axis = Vector2D.rotate90(y_axis);
		
		Vector2D vec = new Vector2D(position, v);
		double x = Vector2D.dot_product(x_axis, vec);
		double y = Vector2D.dot_product(y_axis, vec);
		return new Vector2D(x, y);
	}
	Vector2D get_absolute_coords(Vector2D v){
		Vector2D y_axis = new Vector2D(Math.cos(angle), Math.sin(angle));
		Vector2D x_axis = Vector2D.rotate90(y_axis);
		
		return Vector2D.add(position, Vector2D.add(Vector2D.multiply(x_axis, v.x),Vector2D.multiply(y_axis, v.y)));
	}
	Vector2D get_origin_such_that_local_to_absolute(Vector2D local, Vector2D global){
		Vector2D y_axis = new Vector2D(Math.cos(angle), Math.sin(angle));
		Vector2D x_axis = Vector2D.rotate90(y_axis);
		
		return Vector2D.subtract(global, Vector2D.add(Vector2D.multiply(x_axis, local.x),Vector2D.multiply(y_axis, local.y)));
	}
	
	
	
}
