package world;

import java.awt.*;
import java.util.*;

public class World {
	Panel panel;
	public World(){
		start_timer();
	}
	public void start(Panel _panel){
		initialize();
		
		panel = _panel;
		Thread thread = new Thread(){
			public void run(){
				while(true){
				try{
					sleep(100);
				}
				catch(Exception e){
					e.printStackTrace();
				}
				tick();
				panel.repaint();
				}
			}
		};
		thread.start();
	}
	public void initialize(){
		initialize_map();
		
//		robot = new Robot(new Vector2D(100,100),0);
//	    walls = new ArrayList();
//	    walls.add(new Wall(0,0,600,10));
//	    walls.add(new Wall(0,590,600,600));
//	    walls.add(new Wall(0,0,10,600));
//	    walls.add(new Wall(590,0,600,600));
//	    
//	    task3();
	}
	public void paint(Graphics g){
//		g.setColor(Color.black);
//		g.drawRect(10,10,590,590);
		
		car.paint(g);
		target.paint(g);
		
		for(Wall wall:walls)
			wall.paint(g);
	}
	public boolean pick_robot(Vector2D v){
		//return robot.pick(v);
		Vector2D[] vs = car.get_corners();
		return encloses(vs, v);
	}
	Car car; 
	public void set_steeringwheel_angle(double angle){
		car.steering_angle = angle;
		
		if (car.steering_angle > Math.PI/4)
			car.steering_angle = Math.PI/4;
		if (car.steering_angle < -Math.PI/4)
			car.steering_angle = -Math.PI/4;
	}
	public void set_drive_speed(int speed){
		car.speed = speed;
		
		if (car.speed > 10)
			car.speed = 10;
		if (car.speed < -10)
			car.speed = -10;
	}
	public double get_steeringwheel_angle(){
		return car.steering_angle;
	}
	public double get_drive_speed(){
		return car.speed;
	}
	public Vector2D[] get_car_corners(){
		return car.get_corners();
	}
	public Vector2D get_car_center(){
		Vector2D[] vs = car.get_corners();
		return new Vector2D(
				(vs[0].x+vs[1].x+vs[2].x+vs[3].x)/4,
				(vs[0].y+vs[1].y+vs[2].y+vs[3].y)/4);
	}
	public double get_car_angle(){
		return car.angle;
	}
	
	
	long start_time;
	int time;
	boolean arrived;
	public void start_timer(){
		start_time = System.currentTimeMillis();
		arrived = false;
		time = 0;
	}
	boolean colliding = false;
	public void tick(){
		Vector2D prev_position = new Vector2D(car.position);
		double prev_angle = car.angle;
		
		car.tick();
		
		if (detect_collision(car)){
			if (!colliding)
				java.applet.Applet.newAudioClip(this.getClass().getResource("../bang.wav")).play();
			colliding = true;
			
			car.position = prev_position;
			car.angle = prev_angle;
		}
		else
			colliding = false;

		Vector2D[] corners = target.get_corners();
		Vector2D[] vs = car.get_corners();
		if (encloses(corners, vs[0]) && encloses(corners, vs[1]) && encloses(corners, vs[2]) && encloses(corners, vs[3])
				&& car.speed == 0)
			arrived = true;
		
		if (!arrived) {
			time = (int)(System.currentTimeMillis()-start_time)/1000;
			board.MainPanel.f.setTitle(" Time: "+time);
		}
		else {
			board.MainPanel.f.setTitle(" Time: "+time+"  arrived!");
		}
		
	}
	
	ArrayList<Wall> walls;
	
	boolean detect_collision(Car robot){
		for(Wall wall : walls){
			if (detect_collision(robot, wall))
				return true;
		}
		if (robot.position.x < 10)
			return true;
		if (robot.position.x > 590)
			return true;
		if (robot.position.y < 10)
			return true;
		if (robot.position.y > 590)
			return true;
		return false;
	}
	boolean detect_collision(Car robot, Wall wall){
		Vector2D[] us = robot.get_corners();
		Vector2D[] vs = wall.get_corners();
		
		for(Vector2D v : vs)
			if (encloses(us, v))
				return true;
		
		for(Vector2D u : us)
			if (encloses(vs, u))
				return true;
		
		return false;
	}
	
	
	
	
	boolean encloses(Vector2D[] vs, Vector2D v){
		if (on_the_left(vs[0], vs[1], v))
			return false;
		if (on_the_left(vs[1], vs[2], v))
			return false;
		if (on_the_left(vs[2], vs[3], v))
			return false;
		if (on_the_left(vs[3], vs[0], v))
			return false;
		
		return true;
	}
	boolean on_the_left(Vector2D v0, Vector2D v1, Vector2D v){
		Vector2D vec0 = new Vector2D(v0, v1);
		Vector2D vec = new Vector2D(v0, v);
		return Vector2D.cross_product(vec0, vec)<0;
	}
	
//		for(Wall wall : walls){
//			if (detect_collision(robot, wall))
//				return true;
//		}
//		if (robot.position.x < 10)
//			return true;
//		if (robot.position.x > 590)
//			return true;
//		if (robot.position.y < 10)
//			return true;
//		if (robot.position.y > 590)
//			return true;
//		return false;
//	}
	
	
	
	Target target;
	int world_number = 0;
	public void set_world(int number){
		world_number = number;
		initialize_map();
	}

	public void initialize_map() {
		switch (world_number) {
		case 0: {
			car = new Car(new Vector2D(100, 570), -Math.PI / 2);
			walls = new ArrayList();
			walls.add(new Wall(0, 0, 600, 20));
			walls.add(new Wall(0, 580, 600, 600));
			walls.add(new Wall(0, 0, 20, 600));
			walls.add(new Wall(580, 0, 600, 600));

			target = new Target(430, 410, 140, 160);
			//target = new Target(400, 30, 170, 120);
			break;
		}
		case 1: {
			car = new Car(new Vector2D(30, 400), 0);
			walls = new ArrayList();
			walls.add(new Wall(0, 0, 600, 20));
			walls.add(new Wall(0, 580, 600, 600));
			walls.add(new Wall(0, 0, 20, 600));
			walls.add(new Wall(580, 0, 600, 600));

			walls.add(new Wall(0, 250, 400, 100));

			target = new Target(30, 120, 170, 120);
			break;
		}
		case 2: {
			car = new Car(new Vector2D(80, 30), Math.PI / 2);
			walls = new ArrayList();
			walls.add(new Wall(0, 0, 600, 20));
			walls.add(new Wall(0, 580, 600, 600));
			walls.add(new Wall(0, 0, 20, 600));
			walls.add(new Wall(580, 0, 600, 600));

			walls.add(new Wall(200 - 20, 0, 40, 400));
			walls.add(new Wall(400 - 20, 200, 40, 400));

			target = new Target(430, 430, 140, 140);
			break;
		}
		case 3: {
			car = new Car(new Vector2D(80, 450), Math.PI / 2);
			walls = new ArrayList();
			walls.add(new Wall(0, 0, 600, 20));
			walls.add(new Wall(0, 580, 600, 600));
			walls.add(new Wall(0, 0, 20, 600));
			walls.add(new Wall(580, 0, 600, 600));

			walls.add(new Wall(0, 0, 400, 200));
			walls.add(new Wall(200, 400, 400, 200));

			target = new Target(410, 30, 120, 170);
			break;
		}
		}
	}
}
